# {{ ansible_managed }}
[Unit]
Description=Girder Worker Service
After=network.target

[Service]
Type=simple
User={{ girder_worker_user }}
Group={{ girder_worker_user }}
ExecStart={{ girder_worker_executable }}
ExecStop={{ celery_executable }} multi stopwait worker

[Install]
WantedBy=multi-user.target
